-- ExtendedRaidInfo - by Rinu
-- v1.10

local RaidInfoFrame = RaidInfoFrame

ExtendedRaidInfo = LibStub("AceAddon-3.0"):NewAddon("ExtendedRaidInfo", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ExtendedRaidInfo")
local LGT = LibStub("LibGroupTalents-1.0")

local defaults = {
   profile = {
      savedInstances = {},
      savedPlayers = {},
      savedBosses = {},
      filter = {
         fivePlayersHeroic = true,
         tenPlayers = true,
         tenPlayersHeroic = true,
         twentyfivePlayers = true,
         twentyfivePlayersHeroic = true,
         fourtyPlayers = true,
         expired = true,
         extended = true,
         active = true,
      },
      sort = 2,
   },
}

local function sortdifficulty(a, b) 
   if a.raid > b.raid then 
      return true
   elseif a.raid == b.raid then
      if a.players > b.players then
         return true
      elseif a.players == b.players then
         if a.diff > b.diff then
            return true
         elseif a.diff == b.diff then
            if tonumber(a.expires) > tonumber(b.expires) then
               return true
            elseif a.expires == b.expires then
               if a.name < b.name then
                  return true
               end
            end
         end
      end
   end
      
   return false
end
	
local function sortname(a, b) 
   if a.name < b.name then
      return true
   elseif a.name == b.name then
      if a.raid > b.raid then 
         return true
      elseif a.raid == b.raid then
         if a.players > b.players then
            return true
         elseif a.players == b.players then
            if a.diff > b.diff then
               return true
            elseif a.diff == b.diff then
               if tonumber(a.expires) > tonumber(b.expires) then
                  return true
               end
            end
         end
      end
   end
        
   return false
end

local function sortexpires(a, b) 
   if tonumber(a.expires) > tonumber(b.expires) then
      return true
   elseif tonumber(a.expires) == tonumber(b.expires) then
      if a.raid > b.raid then 
         return true
      elseif a.raid == b.raid then
         if a.players > b.players then
            return true
         elseif a.players == b.players then
            if a.diff > b.diff then
               return true
            elseif a.diff == b.diff then
               if a.name < b.name then
                  return true
               end
            end
         end
      end
   end
      
   return false
end

local classColor = {
   ["DEATHKNIGHT"] = "|cffc41f3b",
   ["DRUID"] = "|cffff7d0a",   
   ["HUNTER"] = "|cffabd473",
   ["MAGE"] = "|cff69ccf0",
   ["PALADIN"] = "|cfff58cba",
   ["PRIEST"] = "|cffffffff",
   ["ROGUE"] = "|cfffff569",
   ["SHAMAN"] = "|cff2459ff",
   ["WARLOCK"] = "|cff9482c9",
   ["WARRIOR"] = "|cffc79c6e",
}

function ExtendedRaidInfo:OnInitialize()
   self.dbi = LibStub("AceDB-3.0"):New("ExtendedRaidInfoDB", defaults, true)
   self.db = self.dbi.profile
    
   -- save the instances every time something might have changed
   self:RegisterEvent("UPDATE_INSTANCE_INFO", "CheckID")
   self:RegisterEvent("RAID_INSTANCE_WELCOME", RequestRaidInfo)   
   self:RegisterEvent("RAID_ROSTER_UPDATE", RequestRaidInfo)
   self:RegisterEvent("PARTY_MEMBERS_CHANGED", RequestRaidInfo)
   self:RegisterEvent("PLAYER_ENTERING_WORLD", RequestRaidInfo)
   
   -- TODO: change the event to something not that frequently
   self:RegisterEvent("PLAYER_REGEN_ENABLED", RequestRaidInfo)
   self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "COMBAT_LOG_EVENT_UNFILTERED") 
   
   -- boss death check
   self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "CHAT_MSG_MONSTER_YELL")
   
   self.topentry = 1
   self.player = UnitName("player")   
   self.selected = nil   
   self.boss = nil
   self.scrollCount = 11
   
   self.classColor = {}
   
   -- save the male class names 
   for k,v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
      self.classColor[v] = classColor[k]
   end
   
   -- save the female class names 
   for k,v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
      self.classColor[v] = classColor[k]
   end
   
   self.notABoss = {
      [37025] = true, -- Stinki
      [37217] = true, -- Precious
   }
      
   ExtendedRaidInfo:CreateMainFrame()   
    
   -- hook the old raid info frame
   UIPanelWindows["RaidInfoFrame"] = nil
   RaidInfoFrame:HookScript("OnShow", self.OnShow)
   
   ExtendedRaidInfo:RegisterChatCommand("extendedraidinfo", function(arg) 
      if arg == "" then
         self.frame:Show()          
      elseif arg == "dbclear" then
         self.db = defaults
      elseif arg == "printc" then
         for k,v in pairs(self.classColor) do
            DEFAULT_CHAT_FRAME:AddMessage(v .. " " .. k)
         end
      elseif arg == "print" then
         for k,v in pairs(self.db.savedInstances[UnitName("player")]) do
            DEFAULT_CHAT_FRAME:AddMessage(v.name .. " " .. v.diffname .. " " .. tostring(v.diff) .. " " .. tostring(v.players))
            
            if self.db.savedPlayers[UnitName("player")][v.id] ~= nil then
               for x,y in pairs(self.db.savedPlayers[UnitName("player")][v.id]) do
                  DEFAULT_CHAT_FRAME:AddMessage(y.name .. " " .. y.class)
               end
            end
            
            if self.db.savedBosses[UnitName("player")][v.id] ~= nil then
               for x,y in pairs(self.db.savedBosses[UnitName("player")][v.id]) do
                  DEFAULT_CHAT_FRAME:AddMessage(x)
               end
            end
         end
      end
   end)
   
   StaticPopupDialogs["CONFIRM_DELETE_ALT_RAIDINFO"] = {
      text = L["Do you really want to delete %s from the list?"],
      button1 = YES,
      button2 = NO,
      OnAccept = function (f) 
         if (self.deleteAlt) then
            self.db.savedInstances[self.deleteAlt] = nil
            self.db.savedPlayers[self.deleteAlt] = nil
            
            self.player = UnitName("player")
            ExtendedRaidInfo:Refresh()
         end
      end,
      OnCancel = function (self) end,
      hideOnEscape = 1,
      timeout = 0,
      exclusive = 1,
      whileDead = 1,
   }
end

function ExtendedRaidInfo:COMBAT_LOG_EVENT_UNFILTERED(...)
   local _, _, event, _, _, _, _, _, guid, name, _, _ = ...
   
   if event == "UNIT_DIED" then
      ExtendedRaidInfo:BossCheck(guid, name)
   end
end

function ExtendedRaidInfo:CHAT_MSG_MONSTER_YELL(_, msg, ...)
   for i = 1, 7 do
      if msg:find(L["deathTrigger" .. i]) then
         -- save instance info
         local instanceName, _, difficulty = GetInstanceInfo()
         
         -- we found a boss!
         self.boss = { name = L["deathTriggerName" .. i], instance = instanceName, difficulty = difficulty }
      end
   end
end

function ExtendedRaidInfo:BossCheck(guid, bossName)
   mobid = tonumber(guid:sub(7, 10), 16)

   if LibStub("LibBossIDs-1.0").BossIDs[mobid] and self.notABoss[mobid] ~= true then
      -- save instance info
      local instanceName, _, difficulty = GetInstanceInfo()
   
      -- we found a boss!
      self.boss = { name = bossName, instance = instanceName, difficulty = difficulty }
   end
end

function ExtendedRaidInfo:OnShow(f)
   RaidInfoFrame:Hide()
   
   -- toggle
   if (ExtendedRaidInfo.window:IsShown()) then
      ExtendedRaidInfo.window:Hide()
   else
      ExtendedRaidInfo.window:Show()
      ExtendedRaidInfo:ShowRaidInfo()
   end
end

function ExtendedRaidInfo:CheckID()
   -- enable raid info button
   if (not _G["RaidFrameRaidInfoButton"]:IsEnabled()) then
      _G["RaidFrameRaidInfoButton"]:Enable()
   end

   local name, instancetype, difficulty = GetInstanceInfo()

   -- get new data
   ExtendedRaidInfo:Refresh()

   -- check if we need to change the player list or the boss list
   if instancetype == "raid" or instancetype == "party" then
      for k,v in pairs(self.db.savedInstances[UnitName("player")]) do
         -- create the boss table
         if self.db.savedBosses[UnitName("player")][v.id] == nil then
            self.db.savedBosses[UnitName("player")][v.id] = {}
         end
      
         -- check if a boss was killed
         if self.boss ~= nil then
            if v.name == self.boss.instance and v.diff == self.boss.difficulty then
               tinsert(self.db.savedBosses[UnitName("player")][v.id], self.boss.name)
               self.boss = nil
            end
         end
      
         -- check if the current instance is one of the saved instances
         if v.name == name and v.diff == difficulty and v.expires > 0 then  
            -- create the player table
            if self.db.savedPlayers[UnitName("player")][v.id] == nil then
               self.db.savedPlayers[UnitName("player")][v.id] = {}
            end
            
            -- get the maximum bosses
            if GetZoneText() == name then            
               _, _, encountersTotal, _ = GetInstanceLockTimeRemaining()
               self.db.savedBosses[UnitName("player")][v.id].total = encountersTotal
            end
            
            -- raid?
            if instancetype == "raid" then
               for i = 1, GetNumRaidMembers() do
                  local playername, _, _, _, class, _, _, _, _, _, _ = GetRaidRosterInfo(i)
                  
                  -- check if we already have that player in the list
                  local found = false
                  
                  for x, y in pairs(self.db.savedPlayers[UnitName("player")][v.id]) do
                     if y.name == playername then
                        found = true
                        
                        -- check talents
                        if (y.spec == nil or type(y.spec) == "number") and UnitInRaid(y.name) then
                           local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(y.name))
                           self.db.savedPlayers[UnitName("player")][v.id][x].spec = spec
                        end
                     end
                  end

                  if found == false then
                     if UnitInRaid(playername) then
                        local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(playername))             
                     else
                        local spec = nil
                     end
                     
                     if class ~= nil then                     
                        tinsert(self.db.savedPlayers[UnitName("player")][v.id], { name = playername, class = class, realm = nil, spec = spec })
                     end
                  end
               end
            else
                  -- check if we already have ourself in the list
                  local found = false
                  
                  for x, y in pairs(self.db.savedPlayers[UnitName("player")][v.id]) do
                     if y.name == UnitName("player") then
                        found = true
                        
                        -- check talents
                        if (y.spec == nil or type(y.spec) == "number") and UnitInParty(y.name) then
                           local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(y.name))
                           self.db.savedPlayers[UnitName("player")][v.id][x].spec = spec
                        end
                     end
                  end
               
                  if found == false then
                     local name, realm = UnitName("player")
                     local class, _ = UnitClass("player")
                     
                     if class ~= nil then     
                        local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(name))                                                 
                        tinsert(self.db.savedPlayers[UnitName("player")][v.id], { name = name, class = class, realm = realm, spec = spec })
                     end
                  end
            
               -- party members
               for i = 1, 4 do
                  -- check if we already have that player in the list
                  local found = false
                  
                  for x, y in pairs(self.db.savedPlayers[UnitName("player")][v.id]) do
                     if y.name == UnitName("party" .. i) then
                        found = true
                        
                        -- check talents
                        if (y.spec == nil or type(y.spec) == "number") and UnitInParty(y.name) then
                           local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(y.name))
                           self.db.savedPlayers[UnitName("player")][v.id][x].spec = spec
                        end
                     end
                  end
               
                  if found == false then
                     local name, realm = UnitName("party" .. i)
                     local class, _ = UnitClass("party" .. i)
                     
                     if class ~= nil then
                        if UnitInParty(name) then
                           local spec, _, _, _ = LGT:GetGUIDTalentSpec(UnitGUID(name))  
                        else
                           local spec = nil
                        end
                        
                        tinsert(self.db.savedPlayers[UnitName("player")][v.id], { name = name, class = class, realm = realm, spec = spec })
                     end
                  end
               end
            end
         end 
      end
   end
end

function ExtendedRaidInfo:Refresh()
   -- refresh the data
   self.db.savedInstances[UnitName("player")] = {}
    
   if self.db.savedPlayers[UnitName("player")] == nil then
      self.db.savedPlayers[UnitName("player")] = {}
   end
   
   if self.db.savedBosses[UnitName("player")] == nil then
      self.db.savedBosses[UnitName("player")] = {}
   end

   local numsaved = GetNumSavedInstances()
   
   if numsaved > 0 then
      for i = 1, numsaved do
         local name, id, expires, diff, locked, extended, _, raid, players, diffname = GetSavedInstanceInfo(i)
         local expires_text = ""
            
         if raid == true then 
            raid = 1 
         else 
            raid = 0 
         end
         
         if expires == 0 then
            expires_text = L["Expired"]
         else        
            -- get the time in words!
            local days, hours, minutes

            days = math.floor(expires / (24 * 60 * 60))                
            hours = math.floor((expires - days * (24 * 60 * 60)) / (60 * 60))                 
            minutes = math.floor((expires - days * (24 * 60 * 60) - hours * (60 * 60)) / 60)

            if days == 1 then
               expires_text = days .. " " .. L["Day"] .. " " 
            elseif days > 1 then
               expires_text = days .. " " .. L["Days"] .. " " 
            end
            
            expires_text = expires_text .. hours .. " " .. L["Hours"] .. " " .. minutes .. " " .. L["Minutes"]
         end
         
         -- localize the diffname
         if diff == 1 then
            diffname = "10 " .. L["Players"]
         elseif diff == 2 then
            diffname = "25 " .. L["Players"]
         elseif diff == 3 then
            diffname = "10 " .. L["Players"] .. " (" .. L["Heroic"] .. ")"
         elseif diff == 4 then
            diffname = "25 " .. L["Players"] .. " (" .. L["Heroic"] .. ")"
         end
            
         if players == 40 then
            diffname = "40 " .. L["Players"]
         elseif players == 5 then
            diffname = "5 " .. L["Players"] .. " (" .. L["Heroic"] .. ")"
         end        

         tinsert(self.db.savedInstances[UnitName("player")], { 
            index = i, 
            id = id, 
            name = name, 
            expires_text = expires_text, 
            expires = expires or 0, 
            diff = diff or 0, 
            players = players or 0, 
            raid = raid or 0, 
            extended = extended, 
            locked = locked, 
            diffname = diffname,
            link = GetSavedInstanceChatLink(i), 
            time = time() 
         })
      end
   end
   
   if self.window:IsShown() then
      ExtendedRaidInfo:ShowRaidInfo()
   end
end

function ExtendedRaidInfo:CreateListItem(header)
   local f = header:CreateFontString(nil,"OVERLAY")
   f:SetHeight(16)
   f:SetFontObject(GameFontNormalSmall)
   f:SetJustifyH("LEFT")
   f:SetTextColor(1, 1, 1, 1)

   return f
end


-- thx to Dridzt for the tooltip update
function ExtendedRaidInfo:ShowTooltip(frame, k)   
	GameTooltip:SetOwner(frame, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", frame, "BOTTOMLEFT")
	GameTooltip:ClearLines()
	
	-- legacy data: only use it if we don't have a .link and it's an alt
	if self.player ~= (UnitName("player")) then -- it's an alt
		if not self.db.savedInstances[self.player][k].link then
	 	 -- legacy data START
			GameTooltip:SetText(self.db.savedInstances[self.player][k].name)
			GameTooltip:AddLine(self.db.savedInstances[self.player][k].diffname, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
			GameTooltip:AddLine("ID: " .. self.db.savedInstances[self.player][k].id .. "\n", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
			-- saved bosses 
			if self.db.savedBosses[self.player] ~= nil and self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id] ~= nil then
				if #self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id] > 0 then
					local complete = #self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id]
					if self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id].total ~= nil then
						total = self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id].total
						GameTooltip:AddLine(L["Killed Bosses"] .. " (" .. complete .. " / " .. total .. "):\n")
					else
						GameTooltip:AddLine(L["Killed Bosses"] .. " (" .. complete .. "):\n")
					end

					for k,v in pairs(self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id]) do
						if k ~= "total" then
							GameTooltip:AddLine(v, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
						end
					end        
					GameTooltip:AddLine("")
				end
			end
	   	-- legacy data END
		else -- we have a link, load it up
			GameTooltip:SetHyperlink(self.db.savedInstances[self.player][k].link)
			GameTooltipTextLeft1:SetText(gsub(GameTooltipTextLeft1:GetText(),strupper(UNKNOWN),self.player))
			-- add the difficulty(?) and ID, still matters for heroic modes, irrelevant for rest
			-- append it to first line to keep it looking similar to legacy data
			GameTooltip:AppendText(HIGHLIGHT_FONT_COLOR_CODE.."\n"..self.db.savedInstances[self.player][k].diffname.."\n".."ID: " .. self.db.savedInstances[self.player][k].id .. "\n" .. FONT_COLOR_CODE_CLOSE)
			-- and add a blank line at the bottom for separation of players list
			GameTooltip:AddLine("")
		end
	else -- it's current player 
		if not self.db.savedInstances[self.player][k].link then -- old data we don't have a link, generate now and store for next lookup
			self.db.savedInstances[self.player][k].link = GetSavedInstanceChatLink(self.db.savedInstances[self.player][k].index)
		end
		GameTooltip:SetHyperlink(self.db.savedInstances[self.player][k].link)
		GameTooltip:AppendText(HIGHLIGHT_FONT_COLOR_CODE.."\n"..self.db.savedInstances[self.player][k].diffname.."\n".."ID: " .. self.db.savedInstances[self.player][k].id .. "\n" .. FONT_COLOR_CODE_CLOSE)
		GameTooltip:AddLine("")
	end
	-- ok, go on and add the stuff we have that's not on the default tooltip
	-- saved players
	if self.db.savedPlayers[self.player] ~= nil and self.db.savedPlayers[self.player][self.db.savedInstances[self.player][k].id] ~= nil then
		if #self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id] > 0 then
			GameTooltip:AddLine(L["Player List"] .. " (" .. #self.db.savedPlayers[self.player][self.db.savedInstances[self.player][k].id] .. "):\n")
	     
			-- sort by alphabet
			table.sort(self.db.savedPlayers[self.player][self.db.savedInstances[self.player][k].id], function (a, b) 
				-- sometimes we get a nil name
				if a.name == nil or b.name == nil then
					return false
				end
				if a.name < b.name then 
					return true 
				else 
					return false
				end 
			end)	

			for k,v in pairs(self.db.savedPlayers[self.player][self.db.savedInstances[self.player][k].id]) do
				if v.name ~= nil and v.class ~= nil then
					local spec = ""
					if v.spec ~= nil and v.spec ~= "No Talent" then
						spec = v.spec
					end
					if spec ~= "" then
						if v.realm == nil then
							GameTooltip:AddDoubleLine(self.classColor[v.class] .. v.name .. "|r", self.classColor[v.class] .. spec .. "|r", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
						else
							GameTooltip:AddDoubleLine(self.classColor[v.class] .. v.name .. " - " .. v.realm .. "|r", self.classColor[v.class] .. spec .. "|r", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
						end
					else
						if v.realm == nil then
							GameTooltip:AddLine(self.classColor[v.class] .. v.name .. "|r", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
						else
							GameTooltip:AddLine(self.classColor[v.class] .. v.name .. " - " .. v.realm .. "|r", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
						end
					end
				end
			end
		end
	end
	
	GameTooltip:Show()
end

function ExtendedRaidInfo:Filter(v)
   if v.players == 5 and self.db.filter.fivePlayersHeroic == false then
      return false
   elseif v.players == 10 and v.diff == 1 and self.db.filter.tenPlayers == false then
      return false
   elseif v.players == 10 and v.diff == 3 and self.db.filter.tenPlayersHeroic == false then
      return false
   elseif v.players == 25 and v.diff == 2 and self.db.filter.twentyfivePlayers == false then
      return false
   elseif v.players == 25 and v.diff == 4 and self.db.filter.twentyfivePlayersHeroic == false then
      return false
   elseif v.players == 40 and self.db.filter.fourtyPlayers == false then
      return false
   elseif v.expires == 0 and self.db.filter.expired == false then
      return false
   elseif v.extended == true and self.db.filter.extended == false then
      return false
   elseif v.expires > 0 and self.db.filter.active == false then
      return false  
   else
      return true
   end   
end

function ExtendedRaidInfo:ShowRaidInfo()
   if self.db.sort == 1 then
      table.sort(self.db.savedInstances[self.player], sortname)	
   elseif self.db.sort == 2 then
      table.sort(self.db.savedInstances[self.player], sortdifficulty)	
   else
      table.sort(self.db.savedInstances[self.player], sortexpires)	
   end

   --[[if self.data ~= nil and #self.data > 0 then
      for i = 1, #self.data do
         self.data[i]:UnregisterAllEvents()
         self.data[i]:Hide()
      end
   end

   self.data = {}]]
   self.selected = nil

   local i = 1
   self.data = self.data or {}   
   
   -- hide bars
   for k,v in pairs(self.data) do
      v:Hide()
   end
   
   -- create the entries for the frame!
   for k,v in pairs(self.db.savedInstances[self.player]) do
      if ExtendedRaidInfo:Filter(v) == true then
         local bar = self.data[i]
         
         if v.expires > 0 and (time() - v.expires) >= v.time then
            self.db.savedInstances[self.player][k].expires = 0
            self.db.savedInstances[self.player][k].expires_text = "Expired"
         end
         
         if (not self.data[i]) then
            bar = CreateFrame("STATUSBAR", "ExtendedRaidInfo_" .. i, self.window, "TextStatusBar")
            bar.name = self:CreateListItem(bar)
            bar.difficulty = self:CreateListItem(bar)
            bar.expires = self:CreateListItem(bar)
            
            bar:SetWidth(410)
            bar:SetHeight(16)
            bar:SetStatusBarTexture("Interface\\AddOns\\ExtendedRaidInfo\\textures\\Armory")
            bar:SetStatusBarColor(0.0, 0.0, 0.0, 0.0)
            
             if #self.data == 0 then
               bar:SetPoint("TOPLEFT", self.header_name, "BOTTOMLEFT", 8, -5)
               bar:SetPoint("TOPRIGHT", self.header_expires, "BOTTOMRIGHT", -4, -5)

               self.data[1] = bar         
            else
               bar:SetPoint("TOPLEFT", self.data[i - 1], "BOTTOMLEFT")
               bar:SetPoint("TOPRIGHT", self.data[i - 1], "BOTTOMRIGHT")

               self.data[i] = bar 
            end
         end
         
         bar.index = i

         bar:SetScript("OnMouseUp", function(f, button)
            if self.selected ~= nil then
               _G[self.selected]:SetStatusBarColor(0.0, 0.0, 0.0, 0.0)
            end
         
            self.selected = f:GetName()
            
            -- post raid lock out on shift click
            if IsShiftKeyDown() then
               if not self.db.savedInstances[self.player][k].link then
                  if self.player == UnitName("player") then                  
                      ChatEdit_InsertLink(GetSavedInstanceChatLink(self.db.savedInstances[self.player][k].index))
                  end
               else
                  ChatEdit_InsertLink(self.db.savedInstances[self.player][k].link)
               end
            end
            
            if self.player == UnitName("player") then
               -- enable extend id for current player
               self.extend:Enable() 
                             
               if self.db.savedInstances[self.player][k].extended then
                  self.extend:SetText(L["Remove Raid Lock Extension"])
               else
                  if self.db.savedInstances[self.player][k].locked then
                     self.extend:SetText(L["Extend Raid Lock"])
                  else
                     self.extend:SetText(L["Reactivate Raid Lock"])
                  end
               end
            end
            
            f:SetStatusBarColor(0.7, 0.7, 0.10, 0.9)    
         end)

         bar:SetScript("OnEnter", function(f)
            f:SetStatusBarColor(0.7, 0.7, 0.10, 0.9)  
            ExtendedRaidInfo:ShowTooltip(f, k) 
         end)

         bar:SetScript("OnLeave", function(f) 
            if self.selected ~= f:GetName() then
               f:SetStatusBarColor(0.0, 0.0, 0.0, 0.0)
            end
            
            GameTooltip:Hide() 
         end)
         
         local complete = -1
         local total = 0
         
         if self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id] ~= nil then
            complete = #self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id]
            total = self.db.savedBosses[self.player][self.db.savedInstances[self.player][k].id].total
         end     

         local color = "abd473"
         if v.expires == 0 then  
            color = "ffffff"
         elseif complete ~= total then
            color = "fff569"
         end

         bar.name:SetText("|cff".. color .. v.name .. "|r")
         bar.name:SetPoint("LEFT", bar, "LEFT")

         bar.expires:SetText("|cff".. color .. v.expires_text .. "|r")
         bar.expires:SetPoint("LEFT", bar, "LEFT", 260, 0)

         bar.difficulty:SetText("|cff".. color .. v.diffname .. "|r")
         bar.difficulty:SetPoint("LEFT", bar, "LEFT", 140, 0)

         if i <= self.scrollCount then
            self.data[i]:Show()
         else
            self.data[i]:Hide()
         end
         
         self.data[i] = bar
         
         i = i + 1
      end
   end	

   self.scroll:SetVerticalScroll(1)
   FauxScrollFrame_Update(self.scroll, #self.data, self.scrollCount, 16)
end

function ExtendedRaidInfo:UpdateScroll()
   if #self.data > 0 then
      FauxScrollFrame_Update(self.scroll, #self.data, self.scrollCount, 16)

      if self.topentry ~= nil and self.topentry ~= 1 then
         self.data[self.topentry]:SetPoint("TOPLEFT", self.data[self.topentry - 1], "BOTTOMLEFT")
         self.data[self.topentry]:SetPoint("TOPRIGHT", self.data[self.topentry - 1], "BOTTOMRIGHT")
      end

      self.data[FauxScrollFrame_GetOffset(self.scroll) + 1]:SetPoint("TOPLEFT", self.header_name, "BOTTOMLEFT", 8, -5)
      self.data[FauxScrollFrame_GetOffset(self.scroll) + 1]:SetPoint("TOPRIGHT", self.header_expires, "BOTTOMRIGHT", -4, -5)

      self.topentry = FauxScrollFrame_GetOffset(self.scroll) + 1

      local x = 0

      for i = 1, #self.data do
         if i > self.topentry - 1 and x < self.scrollCount then
            self.data[i]:Show()
            x = x + 1
         else
            self.data[i]:Hide()
         end
      end
   end
end

function ExtendedRaidInfo:CreateMainFrame()
   self.window = CreateFrame("Frame", "ExtendedRaidInfo", RaidFrame)
   HideUIPanel(ExtendedRaidInfoFrame)

   self.window:SetFrameStrata("DIALOG")
   self.window:SetWidth(410)
   self.window:SetHeight(320)
   self.window:SetPoint("TOPLEFT", RaidFrame, "TOPRIGHT", -40, -10)
   self.window:SetMovable(true)
   self.window:EnableMouse(true)
   self.window:RegisterForDrag("LeftButton")
   self.window:SetScript("OnDragStart", self.window.StartMoving)
   self.window:SetScript("OnDragStop", self.window.StopMovingOrSizing)

   self.titlebg = self.window:CreateTexture(nil, "BORDER")
   self.titlebg:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Title-Background")
   self.titlebg:SetPoint("TOPLEFT", 9, -6)
   self.titlebg:SetPoint("BOTTOMRIGHT", self.window, "TOPRIGHT", -28, -24)

   self.dialogbg = self.window:CreateTexture(nil, "BACKGROUND")
   self.dialogbg:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-CharacterTab-L1")
   self.dialogbg:SetPoint("TOPLEFT", 8, -12)
   self.dialogbg:SetPoint("BOTTOMRIGHT", -6, 8)
   self.dialogbg:SetTexCoord(0.255, 1, 0.29, 1)

   self.topleft = self.window:CreateTexture(nil, "BORDER")
   self.topleft:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.topleft:SetWidth(64)
   self.topleft:SetHeight(64)
   self.topleft:SetPoint("TOPLEFT")
   self.topleft:SetTexCoord(0.501953125, 0.625, 0, 1)

   self.topright = self.window:CreateTexture(nil, "BORDER")
   self.topright:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.topright:SetWidth(64)
   self.topright:SetHeight(64)
   self.topright:SetPoint("TOPRIGHT")
   self.topright:SetTexCoord(0.625, 0.75, 0, 1)

   self.top = self.window:CreateTexture(nil, "BORDER")
   self.top:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.top:SetHeight(64)
   self.top:SetPoint("TOPLEFT", self.topleft, "TOPRIGHT")
   self.top:SetPoint("TOPRIGHT", self.topright, "TOPLEFT")
   self.top:SetTexCoord(0.25, 0.369140625, 0, 1)

   self.bottomleft = self.window:CreateTexture(nil, "BORDER")
   self.bottomleft:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.bottomleft:SetWidth(64)
   self.bottomleft:SetHeight(64)
   self.bottomleft:SetPoint("BOTTOMLEFT")
   self.bottomleft:SetTexCoord(0.751953125, 0.875, 0, 1)

   self.bottomright = self.window:CreateTexture(nil, "BORDER")
   self.bottomright:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.bottomright:SetWidth(64)
   self.bottomright:SetHeight(64)
   self.bottomright:SetPoint("BOTTOMRIGHT")
   self.bottomright:SetTexCoord(0.875, 1, 0, 1)

   self.bottom = self.window:CreateTexture(nil, "BORDER")
   self.bottom:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.bottom:SetHeight(64)
   self.bottom:SetPoint("BOTTOMLEFT", self.bottomleft, "BOTTOMRIGHT")
   self.bottom:SetPoint("BOTTOMRIGHT", self.bottomright, "BOTTOMLEFT")
   self.bottom:SetTexCoord(0.376953125, 0.498046875, 0, 1)

   self.left = self.window:CreateTexture(nil, "BORDER")
   self.left:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.left:SetWidth(64)
   self.left:SetPoint("TOPLEFT", self.topleft, "BOTTOMLEFT")
   self.left:SetPoint("BOTTOMLEFT", self.bottomleft, "TOPLEFT")
   self.left:SetTexCoord(0.001953125, 0.125, 0, 1)

   self.right = self.window:CreateTexture(nil, "BORDER")
   self.right:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.right:SetWidth(64)
   self.right:SetPoint("TOPRIGHT", self.topright, "BOTTOMRIGHT")
   self.right:SetPoint("BOTTOMRIGHT", self.bottomright, "TOPRIGHT")
   self.right:SetTexCoord(0.1171875, 0.2421875, 0, 1)
   
   self.tablebottom = self.window:CreateTexture(nil, "BORDER")
   self.tablebottom:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.tablebottom:SetHeight(64)
   self.tablebottom:SetPoint("BOTTOMLEFT", self.bottomleft, "BOTTOMRIGHT", -55, 33)
   self.tablebottom:SetPoint("BOTTOMRIGHT", self.bottomright, "BOTTOMLEFT", 59, 33)
   self.tablebottom:SetTexCoord(0.376953125, 0.498046875, 0, 1)

   self.close = CreateFrame("Button", nil, self.window, "UIPanelCloseButton")
   self.close:SetPoint("TOPRIGHT", 2, 1)
   self.close:SetScript("OnClick", function() ExtendedRaidInfo.window:Hide() end)
   
   self.altList = CreateFrame("Frame", "ExtendedRaidInfoAltList", self.window, "UIDropDownMenuTemplate")
   self.altList:SetPoint("LEFT", self.topleft, "LEFT", 30, -12)
   
   local function AltInitialize(frame, level)
      -- player
      local info = UIDropDownMenu_CreateInfo()
      
      info.text = UnitName("player")
      info.value = true
      info.func = function(f)
         self.player = UnitName("player")
         self.topentry = nil
         
         ExtendedRaidInfo:ShowRaidInfo()
         
         -- update title
         self.sessionLabel:SetText("Extended Raid Information - " .. UnitName("player"))
         
         -- disable extend button on alt list
         self.extend:Disable()
         
         UIDropDownMenu_SetSelectedID(self.altList, f:GetID())
      end
          
      UIDropDownMenu_AddButton(info, level)
           
      -- alt chars      
      for k,v in pairs(self.db.savedInstances) do
         if #self.db.savedInstances[k] > 0 and k ~= UnitName("player") then
            info = UIDropDownMenu_CreateInfo()
            
            info.text = k
            info.value = false
            info.func = function(f)
               if (not IsModifiedClick()) then
                  self.player = k
                  self.topentry = nil
                  
                  ExtendedRaidInfo:ShowRaidInfo()
                  
                  -- update title
                  self.sessionLabel:SetText("Extended Raid Information - " .. k)
                  
                  -- disable extend button on alt list
                  self.extend:Disable()
                  
                  UIDropDownMenu_SetSelectedID(self.altList, f:GetID())
               else
                  self.deleteAlt = k
                  StaticPopup_Show("CONFIRM_DELETE_ALT_RAIDINFO", k)
               end
            end
                   
            UIDropDownMenu_AddButton(info, level)
         end
      end
   end
   
   self.alttext = self.altList:CreateFontString(nil,"OVERLAY")
   self.alttext:SetHeight(16)
   self.alttext:SetFontObject(GameFontHighlight)
   self.alttext:SetJustifyH("LEFT")
   self.alttext:SetTextColor(1, 1, 1, 1)
   self.alttext:SetPoint("LEFT", self.altList, "LEFT", -15, 2)
   self.alttext:SetText(L["Alt"] .. ":")
   
   UIDropDownMenu_Initialize(self.altList, AltInitialize)
   UIDropDownMenu_SetWidth(self.altList, 120)
   UIDropDownMenu_SetButtonWidth(self.altList, 170)
   UIDropDownMenu_SetSelectedID(self.altList, 1)
   UIDropDownMenu_JustifyText(self.altList, "LEFT")

   --[[
   TODO: create a new calendar entry and invite all saved players (.. to finish the ID or something like that)		

   self.event = CreateFrame("Button", "ERI_Event", self.window, "UIPanelButtonTemplate2")
   self.event:SetPoint("LEFT", self.twinks[#self.twinks], "RIGHT")
   self.event:SetWidth(70)
   self.event:SetText("Add Event")

   self.event:SetScript("OnClick", function(f, button, down)
      if self.selected ~= nil then
         if self.db.savedPlayers[self.player][self.db.savedInstances[self.player][self.selected].id] ~= nil and #self.db.savedPlayers[self.player][self.db.savedInstances[self.player][self.selected].id] > 0 then
            CalendarNewEvent()

            CalendarEventSetTitle(self.db.savedInstances[self.player][self.selected].name)
            CalendarEventSetDescription("")
            DEFAULT_CHAT_FRAME:AddMessage("ERI: New Event: " .. self.db.savedInstances[self.player][self.selected].name)

            local temp = date("*t", time())

            CalendarEventSetDate(temp.month, temp.day, temp.year)
            CalendarEventSetTime(temp.hour, temp.min)

            if self.db.savedInstances[self.player][self.selected].players < 10 then
               CalendarEventSetType(2)
            else
               CalendarEventSetType(1)
            end

            CalendarEventSetTextureID(1)

            for k,v in pairs(self.db.savedPlayers[self.player][self.db.savedInstances[self.player][self.selected].id]) do
               CalendarEventInvite(v.name)
               DEFAULT_CHAT_FRAME:AddMessage("ERI: Inviting " .. v.name)
            end

            CalendarAddEvent()
         end
      end
   end)		
   ]]--
   
   self.extend = CreateFrame("Button", "ExtendedRaidInfoExtend", self.window, "UIPanelButtonTemplate")
   self.extend:SetPoint("LEFT", self.bottomleft, "RIGHT", -52, -8)
   self.extend:SetWidth(strlen(L["Reactivate Raid Lock"]) * 6 + 10)
   self.extend:SetText(L["Extend Raid Lock"])
   self.extend:Disable()
   
   self.extend:SetScript("OnClick", function(f, button, down)
      if self.selected ~= nil then
         local index = _G[self.selected].index
         
         if self.db.savedInstances[self.player][index].extended then
            SetSavedInstanceExtend(self.db.savedInstances[self.player][index].index, false)
         else
            SetSavedInstanceExtend(self.db.savedInstances[self.player][index].index, true)
         end
         
         self.extend:Disable()
         
         RequestRaidInfo()
      end
   end)
   
   self.toggle = CreateFrame("Button", "ExtendedRaidInfoToggle", self.window, "UIPanelButtonTemplate")
   self.toggle:SetPoint("LEFT", self.bottomright, "LEFT", 25, -8)
   self.toggle:SetWidth(30)
   self.toggle:SetText("+")
   
   self.toggle:SetScript("OnClick", function(f, button, down)
      if self.scrollCount == 11 then
         self.window:SetHeight(432)
         self.scrollCount = 18
         
         self.toggle:SetText("-")
      else
         self.window:SetHeight(320)
         self.scrollCount = 11
         
         self.toggle:SetText("+")
      end
      
      ExtendedRaidInfo:ShowRaidInfo()
   end)

   self.sessionLabel = self.window:CreateFontString(nil, "ARTWORK")
   self.sessionLabel:SetJustifyH("CENTER")
   self.sessionLabel:SetFontObject("GameFontNormal")
   self.sessionLabel:SetPoint("CENTER", self.titlebg, "CENTER")
   self.sessionLabel:SetText("Extended Raid Information - " .. UnitName("player"))
   
   self.filter = CreateFrame("Frame", "ExtendedRaidInfoFilter", self.window, "UIDropDownMenuTemplate")
   self.filter:SetPoint("LEFT", self.topright, "LEFT", -132, -12)
   
   local function FilterInitialize(f, level)
      local info = UIDropDownMenu_CreateInfo()
      
      local i = 1
      
      for k,v in pairs(self.db.filter) do
         info = UIDropDownMenu_CreateInfo()
         
         info.text = L[k]
         info.value = v
         info.checked = v
         info.func = function(f)
            local checked = f.checked
            if (type(checked) == "function" ) then
               checked = checked()
            end
         
            if checked == true then
               f.checked = false
            else
               f.checked = true
            end
            
            UIDropDownMenu_Refresh(self.filter, false, 1)
            self.db.filter[k] = f.checked
            
            ExtendedRaidInfo:ShowRaidInfo()
         end
                
         UIDropDownMenu_AddButton(info, level)
         
         i = i + 1
      end
   end
   
   self.filtertext = self.filter:CreateFontString(nil,"OVERLAY")
   self.filtertext:SetHeight(16)
   self.filtertext:SetFontObject(GameFontHighlight)
   self.filtertext:SetJustifyH("LEFT")
   self.filtertext:SetTextColor(1, 1, 1, 1)
   self.filtertext:SetPoint("LEFT", self.filter, "LEFT", -15, 2)
   self.filtertext:SetText(L["Filter"] .. ":")
   
   UIDropDownMenu_Initialize(self.filter, FilterInitialize)
   UIDropDownMenu_SetWidth(self.filter, 150)
   UIDropDownMenu_SetButtonWidth(self.filter, 170)
   UIDropDownMenu_JustifyText(self.filter, "LEFT")

   self.header_name = CreateFrame("Button", "ExtendedRaidInfoHeaderName", self.window, "WhoFrameColumnHeaderTemplate")
   self.header_name:SetScript("OnClick", function() 
      self.db.sort = 1
      ExtendedRaidInfo:ShowRaidInfo()
   end)
   self.header_name:SetPoint("TOPLEFT", self.window, "TOPLEFT", 6, -65)
   self.header_name:SetWidth(140)
   -- fix the background size
   getglobal("ExtendedRaidInfoHeaderNameMiddle"):SetWidth(132)
   self.header_name:SetText(L["Name"])
   self.header_name:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")

   self.header_difficulty = CreateFrame("Button", "ExtendedRaidInfoHeaderDifficulty", self.window, "WhoFrameColumnHeaderTemplate")
   self.header_difficulty:SetScript("OnClick", function() 
      self.db.sort = 2
      ExtendedRaidInfo:ShowRaidInfo()
   end)
   self.header_difficulty:SetPoint("LEFT", self.header_name, "RIGHT")
   self.header_difficulty:SetWidth(120)
   -- fix the background size
   getglobal("ExtendedRaidInfoHeaderDifficultyMiddle"):SetWidth(113)
   self.header_difficulty:SetText(L["Difficulty"])
   self.header_difficulty:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")

   self.header_expires = CreateFrame("Button", "ExtendedRaidInfoHeaderExpires", self.window, "WhoFrameColumnHeaderTemplate")
   self.header_expires:SetScript("OnClick", function() 
      self.db.sort = 3 
      ExtendedRaidInfo:ShowRaidInfo()
   end)
   self.header_expires:SetPoint("LEFT", self.header_difficulty, "RIGHT")
   self.header_expires:SetWidth(110)
   -- fix the background size
   getglobal("ExtendedRaidInfoHeaderExpiresMiddle"):SetWidth(103)
   self.header_expires:SetText(L["Expires"])    
   self.header_expires:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")  

   self.scroll = CreateFrame("ScrollFrame", "ExtendedRaidInfoScroll", self.window, "FauxScrollFrameTemplate")
   self.scroll:SetPoint("TOPLEFT", self.topleft, "TOPLEFT", -35, -95)
   self.scroll:SetPoint("BOTTOMRIGHT", self.bottomright, "TOPRIGHT", -35, -15)
   self.scroll:SetScript("OnVerticalScroll", function(self, offset)
      FauxScrollFrame_OnVerticalScroll(self, offset, 16, function() ExtendedRaidInfo:UpdateScroll() end)
   end)

   self.tabletop = self.window:CreateTexture(nil, "BORDER")
   self.tabletop:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-Border")
   self.tabletop:SetHeight(64)
   self.tabletop:SetPoint("BOTTOMLEFT", self.header_name, "BOTTOMRIGHT", -140, -7)
   self.tabletop:SetPoint("BOTTOMRIGHT", self.header_expires, "BOTTOMLEFT", 140, -7)
   self.tabletop:SetTexCoord(0.376953125, 0.498046875, 0, 1)

   self.window:Hide()		
end
